@@ -5,13 +5,14 @@ use std::io;
 use std::io::Write;
 
 use crossterm::{
-    event::{self, DisableMouseCapture, EnableMouseCapture, Event, KeyCode},
+    event::{self, Event, KeyCode},
     execute,
     terminal::{disable_raw_mode, enable_raw_mode, EnterAlternateScreen, LeaveAlternateScreen},
 };
 use std::cmp::max;
 use std::error::Error;
-use std::time::{Duration, Instant};
+use std::time::Duration;
+use stopwatch::Stopwatch;
 use structopt::StructOpt;
 use tui::{
     backend::{Backend, CrosstermBackend},
@@ -47,39 +48,113 @@ struct App {
     /// Whether the backspace key works while writing
     backspace_active: bool,
 
-    start_time: Instant,
+    writing_time: Stopwatch,
 
     time_goal: Option<i64>,
 
     word_goal: Option<i64>,
+
+    strict_mode: bool,
 }
 
 impl App {
-    fn new(title: String, backspace_active: bool) -> App {
+    fn new(
+        title: String,
+        time_goal: Option<i64>,
+        word_goal: Option<i64>,
+        backspace_active: bool,
+        strict_mode: bool,
+    ) -> App {
         App {
             title,
             text: String::default(),
             input_mode: InputMode::Title,
             backspace_active,
-            time_goal: None,
-            word_goal: None,
-            start_time: Instant::now(),
+            time_goal,
+            word_goal,
+            writing_time: Stopwatch::new(),
+            strict_mode,
         }
     }
 
     fn get_word_count_string(&self) -> String {
         let word_count = self.text.split_whitespace().count();
         match self.word_goal {
-            Some(word_goal) => format!("Word count: {word_count}/{word_goal}"),
-            None => format!("Word count: {word_count}"),
+            Some(word_goal) => format!("{word_count}/{word_goal}"),
+            None => format!("{word_count}"),
         }
     }
 
     fn get_time_string(&self) -> String {
-        let duration = self.start_time.elapsed().as_secs();
+        let duration = self.writing_time.elapsed().as_secs();
         match self.time_goal {
-            Some(time_goal) => format!("Time: {duration}/{time_goal}"),
-            None => format!("Time: {duration}"),
+            Some(time_goal) => format!("{duration} s/{time_goal} s"),
+            None => format!("{duration} s"),
+        }
+    }
+
+    fn get_time_color(&self) -> Color {
+        let duration = self.writing_time.elapsed().as_secs();
+        match self.time_goal {
+            Some(i) => {
+                if duration as i64 >= i {
+                    Color::Green
+                } else {
+                    Color::Yellow
+                }
+            }
+            None => Color::DarkGray,
+        }
+    }
+
+    fn get_word_count_color(&self) -> Color {
+        let word_count = self.text.split_whitespace().count();
+        match self.word_goal {
+            Some(i) => {
+                if word_count as i64 >= i {
+                    Color::Green
+                } else {
+                    Color::Yellow
+                }
+            }
+            None => Color::DarkGray,
+        }
+    }
+
+    fn achieved_goals(&self) -> bool {
+        let word_goal_achieved = match self.word_goal {
+            Some(i) => self.text.split_whitespace().count() as i64 >= i,
+            None => true,
+        };
+        let time_goal_achieved = match self.time_goal {
+            Some(i) => self.writing_time.elapsed().as_secs() as i64 >= i,
+            None => true,
+        };
+        word_goal_achieved && time_goal_achieved
+    }
+
+    fn get_title(&self) -> Vec<Span> {
+        match self.input_mode {
+            InputMode::Title => vec![
+                Span::raw("Press "),
+                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to exit, "),
+                Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to start the writing session."),
+            ],
+            InputMode::Writing => {
+                if self.strict_mode && !self.achieved_goals() {
+                    vec![Span::raw(
+                        "Keep writing until you achieve your writing goal! ",
+                    )]
+                } else {
+                    vec![
+                        Span::raw("Press "),
+                        Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
+                        Span::raw(" to stop writing"),
+                    ]
+                }
+            }
         }
     }
 }
@@ -129,26 +204,27 @@ fn main() -> Result<(), Box<dyn Error>> {
     if let Some(0) = word_goal {
         word_goal = None;
     }
+    let strict_mode = settings.get_bool("strict_mode").unwrap_or(true);
 
     // setup terminal
     enable_raw_mode()?;
     let mut stdout = io::stdout();
-    execute!(stdout, EnterAlternateScreen, EnableMouseCapture)?;
+    execute!(stdout, EnterAlternateScreen)?;
     let backend = CrosstermBackend::new(stdout);
     let mut terminal = Terminal::new(backend)?;
 
-    let mut app = App::new(title.to_string(), backspace_active);
-    app.time_goal = time_goal;
-    app.word_goal = word_goal;
+    let mut app = App::new(
+        title.to_string(),
+        time_goal,
+        word_goal,
+        backspace_active,
+        strict_mode,
+    );
     let res = run_app(&mut terminal, &mut app);
 
     // restore terminal
     disable_raw_mode()?;
-    execute!(
-        terminal.backend_mut(),
-        LeaveAlternateScreen,
-        DisableMouseCapture
-    )?;
+    execute!(terminal.backend_mut(), LeaveAlternateScreen)?;
     terminal.show_cursor()?;
 
     if !app.text.is_empty() {
@@ -180,6 +256,7 @@ fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: &mut App) -> io::Res
                 match app.input_mode {
                     InputMode::Title => match key.code {
                         KeyCode::Enter => {
+                            app.writing_time.start();
                             app.input_mode = InputMode::Writing;
                         }
                         KeyCode::Esc => {
@@ -204,7 +281,10 @@ fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: &mut App) -> io::Res
                             }
                         }
                         KeyCode::Esc => {
-                            app.input_mode = InputMode::Title;
+                            if app.achieved_goals() || !app.strict_mode {
+                                app.writing_time.stop();
+                                app.input_mode = InputMode::Title;
+                            }
                         }
                         _ => {}
                     },
@@ -229,28 +309,7 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
         )
         .split(f.size());
 
-    let (msg, style) = match app.input_mode {
-        InputMode::Title => (
-            vec![
-                Span::raw("Press "),
-                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to exit, "),
-                Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to start writing."),
-            ],
-            Style::default().add_modifier(Modifier::RAPID_BLINK),
-        ),
-        InputMode::Writing => (
-            vec![
-                Span::raw("Press "),
-                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to stop writing"),
-            ],
-            Style::default(),
-        ),
-    };
-    let mut text = Text::from(Spans::from(msg));
-    text.patch_style(style);
+    let text = Text::from(Spans::from(app.get_title()));
     let help_message = Paragraph::new(text);
     f.render_widget(help_message, chunks[0]);
 
@@ -298,13 +357,24 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
 
     let text = Paragraph::new(wrapped_text)
         .style(Style::default().fg(widget_colors.1))
-        .block(Block::default().borders(Borders::ALL).title("Title"));
+        .block(Block::default().borders(Borders::ALL).title("Text"));
     f.render_widget(text, chunks[2]);
 
-    let stats = format!("{} {}", app.get_word_count_string(), app.get_time_string());
-    let stats = Paragraph::new(stats)
-        .style(Style::default().fg(Color::DarkGray))
-        .block(Block::default().borders(Borders::ALL).title("Statistics"))
-        .wrap(Wrap { trim: true });
-    f.render_widget(stats, chunks[3]);
+    let stat_chunks = Layout::default()
+        .direction(Direction::Horizontal)
+        .constraints([Constraint::Percentage(50), Constraint::Percentage(50)].as_ref())
+        .split(chunks[3]);
+    {
+        let stats = Paragraph::new(app.get_word_count_string())
+            .style(Style::default().fg(app.get_word_count_color()))
+            .block(Block::default().borders(Borders::ALL).title("Word count"))
+            .wrap(Wrap { trim: true });
+        f.render_widget(stats, stat_chunks[0]);
+
+        let stats = Paragraph::new(app.get_time_string())
+            .style(Style::default().fg(app.get_time_color()))
+            .block(Block::default().borders(Borders::ALL).title("Time"))
+            .wrap(Wrap { trim: true });
+        f.render_widget(stats, stat_chunks[1]);
+    }
 }
